<?php
/**
* @copyright	Copyright (C) 2009 - 2012 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	Frontend
* @contact 		payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

require_once("api/Parameters.php");
require_once("api/ClassOrder.php");
require_once("api/ClassTerminal.php");
require_once("api/ClassCustomer.php");
require_once("api/ClassRegisterRequest.php");
require_once("api/ClassEnvironment.php");
require_once("api/ClassRecurring.php");
require_once("api/ClassItem.php");
require_once("api/ClassArrayOfItem.php");
require_once("api/ClassProcessRequest.php");

class PayplansAppNets extends PayplansAppPayment
{
	protected $_location	= __FILE__;
	
	function isApplicable($refObject = null, $eventName='')
	{
		// return true for event onPayplansSubscriptionBeforeSave
		if($eventName == 'onPayplansSubscriptionBeforeSave'){
			return true;
		}
		
		return parent::isApplicable($refObject, $eventName);
	}
		
    function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{		
		$invoice    = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount     = intval( $invoice->getTotal());
		$paymentKey = $payment->getKey();
		
		$merchantId = $this->getAppParam('merchantId','');
		$token      = $this->getAppParam('token','');
		$test       = $this->getAppParam('test','');
		
		$terminalUrl= 'https://epayment.bbs.no/terminal/default.aspx';
		
		if($test){
			$terminalUrl = 'https://epayment-test.bbs.no/terminal/default.aspx';
		}
		
		$root = JURI::root();
    	if(XiFactory::getConfig()->https == true){
    		$root = JString::str_ireplace("http:", "https:", $root);
    	}
		  
    	$redirect_url		  = $root.'index.php?option=com_payplans&view=payment&task=complete&payment_key='.$payment->getKey();
		$firstRecurring		  = false;
		
    	if($invoice->isRecurring()){
    		$firstRecurring = true;
    	}
    	
    	$registerResult = $this->doRegister($invoice, $redirect_url,'B', $firstRecurring);
		$terminalUrl	= $terminalUrl.'?merchantId='.$merchantId.'&transactionId='.$registerResult->TransactionId;
  		$cancelUrl 		= $root.'index.php?option=com_payplans&view=payment&task=complete&action=cancel&payment_key='.$payment->getKey();

		$this->assign('cancelUrl', $cancelUrl);
		$this->assign('terminalUrl', $terminalUrl);
	
	  	return $this->_render('form');
	}
	
	
	
	function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{		
		if($action == 'cancel'){
			return true;
		}
		if($data['responseCode'] != 'OK')
		{
			$message 			= XiText::_('COM_PAYPLANS_LOGGER_NETS_ERROR');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
			$action	= 'error';
		    return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
			
		}
		
		$invoice 	= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$amount 	= $invoice->getTotal();
		
		//in case of recurring
		$recurring = $invoice->isRecurring();
		if($recurring){
			$ProcessResult 	= $this->doProcess('SALE',$data['transactionId']);
			$result 		= $this->doQuery($data);
			$payment->setParam('PanHash', $result->CardInformation->PanHash)->save();
		}
		else{
			$ProcessResult = $this->doProcess('AUTH',$data['transactionId']);
		}
		
  		if ($ProcessResult->ResponseCode == "OK")
  		{
	  		// if same notification came more than one time
			// Check if transaction already exists,if yes then do nothing and return
			$txn_id 	= isset($data['transactionId']) ? $data['transactionId'] : 0; 
			$subscr_id	= isset($data['transactionId']) ? $data['transactionId'] : 0;
			$transactions 	= $this->_getExistingTransaction($invoice->getId(), $txn_id, $subscr_id, 0);
			if(!empty($transactions)) {
    			return true;
	    	}
  			
  			// do this for normal payment
			// save transaction for normal payment
			$transaction = PayplansTransaction::getInstance();
			$transaction->set('user_id', $invoice->getBuyer())
						->set('invoice_id', $invoice->getId())
						->set('payment_id', $payment->getId())
						->set('amount', $amount)
						->set('gateway_txn_id',$txn_id )
						->set('gateway_subscr_id',$subscr_id )
						->set('message', 'COM_PAYPLANS_PAYMENT_APP_NETS_PAYMENT_COMPLETED_SUCCESSFULLY')
						->set('params', PayplansHelperParam::arrayToIni($data))
						->save();
  		}
  		else{
  			$message 			= XiText::_('COM_PAYPLANS_LOGGER_NETS_ERROR');
			PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $data, 'PayplansPaymentFormatter', '', true);
			$action	= 'error';
  		}
  		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);
	}
	
	public function processPayment(PayplansPayment $payment, $invoiceCount)
	{
		$invoice 	= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$redirect_url 	= null;
		$amount 	= $invoice->getTotal();
		
		####  PARAMETERS IN RECURRING  ####
		$ExpiryDate           = null; // Required (if type R
		$Frequency            = null; // Required (if type R)
		$Type                 = "R"; // Optional parameter (unless Pan Hash is supplied, then it is required)
		$PanHash              = $payment->getParam('PanHash'); // Optional paramete
		
		####  RECURRING OBJECT  ####
		$Recurring = new Recurring(
		        $ExpiryDate,
		        $Frequency,
		        $Type,
		        $PanHash
		);

		//register with type 'C' for next payment
		$registerResult = $this->doRegister($invoice, $redirect_url, 'C', false, $Recurring);
				
		if(!isset($registerResult->TransactionId)){
			return false;
		}
		
		$transactionId = $registerResult->TransactionId;
		$processResult = $this->doProcess('SALE', $transactionId);
		

		if($processResult->ResponseCode != "OK"){
			return false;
		}
		
		$transaction = PayplansTransaction::getInstance();
		$transaction->set('user_id', $invoice->getBuyer())
					->set('invoice_id', $invoice->getId())
					->set('payment_id', $payment->getId())
					->set('amount', $amount)
					->set('gateway_txn_id', $transactionId)
					->set('gateway_subscr_id', $transactionId)
					->set('params', PayplansHelperParam::arrayToIni($processResul))
					->set('message', 'COM_PAYPLANS_PAYMENT_APP_NETS_PAYMENT_COMPLETED_SUCCESSFULLY')
					->save();		
	}	
	
	public function doProcess($type='AUTH',$transactionId)
	{
		$description = "";
		$operation = $type;
		$transactionAmount ="";
		$transactionReconRef = "";
		//$transactionId = $data['transactionId'];
		
		$merchantId = $this->getAppParam('merchantId','');
		$token      = $this->getAppParam('token','');
		
		####  PROCESS OBJECT  ####
		$ProcessRequest = new ProcessRequest(
		  $description,
		  $operation,
		  $transactionAmount,
		  $transactionId,
		  $transactionReconRef
		);

		$InputParametersOfProcess = array
		(
		  "token"       => $token,
		  "merchantId"  => $merchantId,
		  "request"     => $ProcessRequest 
		);
		
		$wsdl		= 'https://epayment.bbs.no/Netaxept.svc?wsdl';
		if($this->getAppParam('test')){
			$wsdl	     = 'https://epayment-test.bbs.no/netaxept.svc?wsdl';
		}
		// Creating new client without proxy
  		$client = new SoapClient($wsdl, array('trace' => true,'exceptions' => true ));
  		
  		$OutputParametersOfProcess = $client->__call('Process' , array("parameters"=>$InputParametersOfProcess));
  		return $OutputParametersOfProcess->ProcessResult;
	}
	
	public function doQuery($data)
	{
		require_once("api/ClassQueryRequest.php");	
		
		$merchantId    = $this->getAppParam('merchantId','');
		$token         = $this->getAppParam('token','');
		$transactionId = $data['transactionId'];
		
		####  QUERY OBJECT  ####
		$QueryRequest = new QueryRequest(
		  $transactionId
		); 
		
		####  ARRAY WITH QUERY PARAMETERS  ####
		$InputParametersOfQuery = array
		(
		  "token"       => $token,
		  "merchantId"  => $merchantId,
		  "request"     => $QueryRequest 
		);
		
		$wsdl		= 'https://epayment.bbs.no/Netaxept.svc?wsdl';
		if($this->getAppParam('test')){
			$wsdl	     = 'https://epayment-test.bbs.no/netaxept.svc?wsdl';
		}
		
		 // Creating new client without proxy
 		$client = new SoapClient($wsdl, array('trace' => true,'exceptions' => true ));
 		
		$OutputParametersOfQuery = $client->__call('Query' , array("parameters"=>$InputParametersOfQuery));
  		return $OutputParametersOfQuery->QueryResult; 
	}
	
	//type B: Hosted
	function doRegister($invoice, $redirect_url,$type = 'B', $firstRecurring = false, $Recurring = null)
	{
		$merchantId = $this->getAppParam('merchantId','');
		$token      = $this->getAppParam('token','');
		$amount     = intval( $invoice->getTotal())*100; //in nets its required to multiple amount with 100
		
		$wsdl		= 'https://epayment.bbs.no/Netaxept.svc?wsdl';
		if($this->getAppParam('test')){
			$wsdl	     = 'https://epayment-test.bbs.no/netaxept.svc?wsdl';
		}
		
		$user				  = $invoice->getBuyer(PAYPLANS_INSTANCE_REQUIRE);
		$currencyCode         = $invoice->getCurrency('isocode');
		$force3DSecure        = null;   // Optional parameter
		$orderNumber          = $invoice->getId();
		$UpdateStoredPaymentInfo = null;
		
		####  PARAMETERS IN ENVIRONMENT  ####
		$Language             = null; // Optional parameter
		$OS                   = null; // Optional parameter
		$WebServicePlatform   = 'PHP5'; // Required (for Web Services)
		
		####  PARAMETERS IN TERMINAL  ####
		$autoAuth             = null; // Optional parameter
		$paymentMethodList    = null; // Optional parameter
		$language             = 'en_GB'; // Optional parameter
		$orderDescription     = null; // Optional parameter
		$redirectOnError      = null; // Optional parameter
		
		####  PARAMETERS IN REGISTER REQUEST  ####
		$AvtaleGiro           = null; // Optional parameter
		$CardInfo             = null; // Optional parameter
		$Customer             = null; // Optional parameter
		$description          = null; // Optional parameter
		$DnBNorDirectPayment  = null; // Optional parameter
		$Environment          = null; // Optional parameter for REST
		$MicroPayment         = null; // Optional parameter
		$serviceType          = $type;// Optional parameter
		//$Recurring            = null; // Optional parameter
		$transactionId        = null; // Optional parameter
		$transactionReconRef  = null; // Optional parameter
		
		####  PARAMETERS IN CUSTOMER  ####
		$customerAddress1                   = null;    // Optional parameter (required if DnBNorDirectPayment)
		$customerAddress2                   = null;    // Optional parameter
		$customerCompanyName                = null;    // Optional parameter
		$customerCompanyRegistrationNumber  = null;    // Optional parameter
		$customerCountry                    = $user->getCountry();   // Optional parameter
		$customerFirstName                  = null;    // Optional parameter (required if DnBNorDirectPayment)
		$customerLastName                   = null;    // Optional parameter (required if DnBNorDirectPayment)
		$customerNumber                     = $user->getId();       // Optional parameter
		$customerEmail                      = $user->getEmail();    // Optional parameter
		$customerPhoneNumber                = null;    // Optional parameter
		$customerPostcode                   = null;    // Optional parameter (required if DnBNorDirectPayment)
		$customerSocialSecurityNumber       = null;    // Optional parameter 
		$customerTown                       = null;    // Optional parameter (required if DnBNorDirectPayment)
		
		
		$ArrayOfItem = null; // no goods for Klana ==> normal transaction

		if(empty($Recurring) && $firstRecurring)
		{
			$time =	$invoice->getExpiration(PAYPLANS_RECURRING);
			
			$count = $recurCount = $invoice->getRecurrenceCount();
			$expirationRaw = $invoice->getExpiration(PAYPLANS_RECURRING, true);
			$recurCount--;
			//calculate end date
	    	$endDate	   = new XiDate();
	    	if($recurCount != 0){
		    	for($recurCount; $recurCount > 0; $recurCount--){
		    			$endDate = $endDate->addExpiration($expirationRaw);				
		    	}
	    	}
		    $endDate	=$endDate->toFormat('%Y%m%d');
		  
			####  PARAMETERS IN RECURRING  ####
			$ExpiryDate           = $endDate; //Required (if type R
			$Frequency            = --$count;  // Required (if type R)
			$Type                 = "R";  // Optional parameter (unless Pan Hash is supplied, then it is required)
			$PanHash              = null; // Optional parameter
		
			####  RECURRING OBJECT  ####	
			$Recurring = new Recurring(
			        $ExpiryDate,
			        $Frequency,
			        $Type,
			        $PanHash
			);
 
		}
		
		####  ENVIRONMENT OBJECT  ####
		$Environment = new Environment(
		  $Language,
		  $OS,
		  $WebServicePlatform 
		);
		
		####  TERMINAL OBJECT  ####
		$Terminal = new Terminal(
		  $autoAuth,
		  $paymentMethodList,
		  $language,
		  $orderDescription,
		  $redirectOnError,
		  $redirect_url  	
		);
		
		$ArrayOfItem = null; // no goods for Klana ==> normal transaction
		
		####  ORDER OBJECT  ####
		$Order = new Order(
		  $amount,
		  $currencyCode,
		  $force3DSecure,
		  $ArrayOfItem,
		  $orderNumber,
		  $UpdateStoredPaymentInfo
		);
		
		####  CUSTOMER OBJECT  ####
		$Customer = new Customer(
		  $customerAddress1,
		  $customerAddress2,
		  $customerCompanyName,
		  $customerCompanyRegistrationNumber,
		  $customerCountry,
		  $customerNumber,
		  $customerEmail,
		  $customerFirstName,
		  $customerLastName,
		  $customerPhoneNumber,
		  $customerPostcode,
		  $customerSocialSecurityNumber,
		  $customerTown
		);
		
		####  START REGISTER REQUEST  ####
		$RegisterRequest = new RegisterRequest(
		  $AvtaleGiro,
		  $CardInfo,
		  $Customer,
		  $description,
		  $DnBNorDirectPayment,
		  $Environment,
		  $MicroPayment,
		  $Order,
		  $Recurring,
		  $serviceType,
		  $Terminal,
		  $transactionId,
		  $transactionReconRef
		);


		####  ARRAY WITH REGISTER PARAMETERS  ####
		$InputParametersOfRegister = array
		(
		        "token"                 => $token,
		        "merchantId"            => $merchantId,
		        "request"               => $RegisterRequest
		);
		
		// Creating new client without proxy
		$client = new SoapClient($wsdl, array('trace' => true,'exceptions' => true ));
		$OutputParametersOfRegister = $client->__call('Register' , array("parameters"=>$InputParametersOfRegister));
		    
		// RegisterResult
		return $OutputParametersOfRegister->RegisterResult; 
	}
}